@@ -42,7 +42,7 @@
 
 #ifndef lint
 static const char rcsid[] _U_ =
-     "@(#) $Header: /tcpdump/master/tcpdump/print-ascii.c,v 1.16 2004/07/21 22:00:10 guy Exp $";
+     "@(#) $Header: /tcpdump/master/tcpdump/print-ascii.c,v 1.17 2005-07-06 20:53:32 guy Exp $";
 #endif
 #include <tcpdump-stdinc.h>
 #include <stdio.h>
@@ -57,78 +57,74 @@ static const char rcsid[] _U_ =
 		(HEXDUMP_HEXSTUFF_PER_SHORT * HEXDUMP_SHORTS_PER_LINE)
 
 void
-ascii_print_with_offset(register const char *ident, register const u_char *cp, register u_int length,
-			register u_int oset)
+ascii_print(register const u_char *cp, register u_int length)
+{
+	register int s;
+
+	putchar('\n');
+	while (length > 0) {
+		s = *cp++;
+		length--;
+		if (!isgraph(s) &&
+		    (s != '\t' && s != ' ' && s != '\n' && s != '\r'))
+			putchar('.');
+		else
+			putchar(s);
+	}
+}
+
+void
+hex_and_ascii_print_with_offset(register const char *ident,
+    register const u_char *cp, register u_int length, register u_int oset)
 {
 	register u_int i;
 	register int s1, s2;
 	register int nshorts;
 	char hexstuff[HEXDUMP_SHORTS_PER_LINE*HEXDUMP_HEXSTUFF_PER_SHORT+1], *hsp;
 	char asciistuff[ASCII_LINELENGTH+1], *asp;
-	u_int maxlength = (Aflag ? ASCII_LINELENGTH : HEXDUMP_SHORTS_PER_LINE);
 
 	nshorts = length / sizeof(u_short);
 	i = 0;
 	hsp = hexstuff; asp = asciistuff;
-	if (Aflag) *(asp++) = '\n';
 	while (--nshorts >= 0) {
 		s1 = *cp++;
 		s2 = *cp++;
-		if (Aflag) {
-			i += 2;
-			*(asp++) = (isgraph(s1) ? s1 : (s1 != '\t' && s1 != ' ' && s1 != '\n' && s1 != '\r' ? '.' : s1) );
-			*(asp++) = (isgraph(s2) ? s2 : (s2 != '\t' && s2 != ' ' && s2 != '\n' && s2 != '\r' ? '.' : s2) );
-			if (s1 == '\n' || s2 == '\n') i = maxlength;
-
-		} else {
-			(void)snprintf(hsp, sizeof(hexstuff) - (hsp - hexstuff),
-			    " %02x%02x", s1, s2);
-			hsp += HEXDUMP_HEXSTUFF_PER_SHORT;
-			*(asp++) = (isgraph(s1) ? s1 : '.');
-			*(asp++) = (isgraph(s2) ? s2 : '.');
-			i++;
-		}
-		if (i >= maxlength) {
+		(void)snprintf(hsp, sizeof(hexstuff) - (hsp - hexstuff),
+		    " %02x%02x", s1, s2);
+		hsp += HEXDUMP_HEXSTUFF_PER_SHORT;
+		*(asp++) = (isgraph(s1) ? s1 : '.');
+		*(asp++) = (isgraph(s2) ? s2 : '.');
+		i++;
+		if (i >= HEXDUMP_SHORTS_PER_LINE) {
 			*hsp = *asp = '\0';
-			if (Aflag) {
-				(void)printf("%s", asciistuff);
-			} else {
-				(void)printf("%s0x%04x: %-*s  %s",
-				    ident, oset, HEXDUMP_HEXSTUFF_PER_LINE,
-				    hexstuff, asciistuff);
-			}
+			(void)printf("%s0x%04x: %-*s  %s",
+			    ident, oset, HEXDUMP_HEXSTUFF_PER_LINE,
+			    hexstuff, asciistuff);
 			i = 0; hsp = hexstuff; asp = asciistuff;
 			oset += HEXDUMP_BYTES_PER_LINE;
 		}
 	}
 	if (length & 1) {
 		s1 = *cp++;
-		if (Aflag) {
-			*(asp++) = (isgraph(s1) ? s1 : (s1 != '\t' && s1 != ' ' && s1 != '\n' && s1 != '\r' ? '.' : s1) );
-		} else {
-			(void)snprintf(hsp, sizeof(hexstuff) - (hsp - hexstuff),
-			    " %02x", s1);
-			hsp += 3;
-			*(asp++) = (isgraph(s1) ? s1 : '.');
-		}
+		(void)snprintf(hsp, sizeof(hexstuff) - (hsp - hexstuff),
+		    " %02x", s1);
+		hsp += 3;
+		*(asp++) = (isgraph(s1) ? s1 : '.');
 		++i;
 	}
 	if (i > 0) {
 		*hsp = *asp = '\0';
-		if (Aflag) {
-			(void)printf("%s%s", ident, asciistuff);
-		} else {
-			(void)printf("%s0x%04x: %-*s  %s",
-			     ident, oset, HEXDUMP_HEXSTUFF_PER_LINE,
-			     hexstuff, asciistuff);
-		}
+		(void)printf("%s0x%04x: %-*s  %s",
+		     ident, oset, HEXDUMP_HEXSTUFF_PER_LINE,
+		     hexstuff, asciistuff);
 	}
 }
 
 void
-ascii_print(register const char *ident, register const u_char *cp, register u_int length)
+hex_and_ascii_print(register const char *ident, register const u_char *cp,
+    register u_int length)
 {
-	ascii_print_with_offset(ident, cp, length, 0);
+	hex_and_ascii_print_with_offset(ident, cp, length, 0);
 }
 
 /*
@@ -171,15 +167,17 @@ hex_print(register const char *ident, register const u_char *cp, register u_int
 int
 main(int argc, char *argv[])
 {
-	hex_print("Hello, World!\n", 14);
+	hex_print("\n\t", "Hello, World!\n", 14);
+	printf("\n");
+	hex_and_ascii_print("\n\t", "Hello, World!\n", 14);
 	printf("\n");
 	ascii_print("Hello, World!\n", 14);
 	printf("\n");
 #define TMSG "Now is the winter of our discontent...\n"
-	ascii_print_with_offset(TMSG, sizeof(TMSG) - 1, 0x100);
+	hex_print_with_offset("\n\t", TMSG, sizeof(TMSG) - 1, 0x100);
+	printf("\n");
+	hex_and_ascii_print_with_offset("\n\t", TMSG, sizeof(TMSG) - 1, 0x100);
 	printf("\n");
 	exit(0);
 }
 #endif /* MAIN */
-
-
