@@ -394,9 +394,6 @@ trunc2:
 static int dccp_print_option(const u_char *option)
 {	
 	u_int8_t optlen, i;
-	u_int32_t *ts;
-	u_int16_t *var16;
-	u_int32_t *var32;
 
 	TCHECK(*option);
 
@@ -470,22 +467,17 @@ static int dccp_print_option(const u_char *option)
 		for (i = 0; i < optlen -2; i ++) printf("%02x", *(option +2 + i));	
 		break;
 	case 41:
-		ts = (u_int32_t *)(option + 2);
-		printf("timestamp %u", (u_int32_t)ntohl(*ts));
+		printf("timestamp %u", EXTRACT_32BITS(option + 2));
 		break;
 	case 42:
-		ts = (u_int32_t *)(option + 2);
-		printf("timestamp_echo %u", (u_int32_t)ntohl(*ts));
+		printf("timestamp_echo %u", EXTRACT_32BITS(option + 2));
 		break;
 	case 43:
 		printf("elapsed_time ");
-		if (optlen == 6){
-			ts = (u_int32_t *)(option + 2);
-			printf("%u", (u_int32_t)ntohl(*ts));
-		} else {
-			var16 = (u_int16_t *)(option + 2);
-			printf("%u", ntohs(*var16));
-		}	
+		if (optlen == 6)
+			printf("%u", EXTRACT_32BITS(option + 2));
+		else
+			printf("%u", EXTRACT_16BITS(option + 2));
 		break;
 	case 44:
 		printf("data_checksum ");
@@ -496,12 +488,10 @@ static int dccp_print_option(const u_char *option)
 			printf("CCID option %d",*option);
 			switch (optlen) {
 				case 4:
-					var16 = (u_int16_t *)(option + 2);
-					printf(" %u",ntohs(*var16));
+					printf(" %u", EXTRACT_16BITS(option + 2));
 					break;
 				case 6:
-					var32 = (u_int32_t *)(option + 2);
-					printf(" %u",(u_int32_t)ntohl(*var32));
+					printf(" %u", EXTRACT_32BITS(option + 2));
 					break;
 				default:
 					break;
