@@ -55,6 +55,10 @@
 #include <sys/types.h>
 #include <net/netdb.h>  /* in wpcap's Win32/include */
 
+#ifndef NBBY
+#define NBBY	8
+#endif
+
 #if !defined(__MINGW32__) && !defined(__WATCOMC__)
 #undef toascii
 #define isascii __isascii
