@@ -3,7 +3,7 @@
 uudecode lmp.puu
 
 echo -n test lmp ...
-../tcpdump -t -n -v -r lmp.pcap >lmp.new
+../tcpdump -t -n -v -v -v -r lmp.pcap >lmp.new
 if diff lmp.new lmp.out
 then
 	echo passed.
